/***
 * Excerpted from "No Fluff Just Stuff 2006 Anthology"
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/nfjs06 for more book information.
***/
package net.bosatsu.util.regex.cc;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.regex.Pattern;

import junit.framework.TestCase;
import net.bosatsu.util.regex.account.AccountRegularExpressionTransformer;
import net.bosatsu.util.regex.io.RegularExpressionInputStream;
import net.bosatsu.util.regex.io.RegularExpressionReplacer;

/**
 * Unit test of RegexInputStream classes.
 * @author bsletten
 *
 */
public class CCRegularExpressionInputStream_TEST extends TestCase {
    private String dataFile;

    public void setUp() {
        dataFile = System.getProperty("user.dir")
            + File.separator + "data" + File.separator + "test.dat";
    }

    public void testBasic() {
        RegularExpressionReplacer rer = new RegularExpressionReplacer();
        rer.addPatternMapping( Pattern.compile( "\\b(\\d{16})\\b" ), new CCMaskRegularExpressionTransformer() );

        runTest( dataFile, rer );
    }

    public void testAccount() {
        RegularExpressionReplacer rer = new RegularExpressionReplacer();
        rer.addPatternMapping( Pattern.compile( "\\b(\\d{16})\\b" ), new CCMaskRegularExpressionTransformer() );
        rer.addPatternMapping( Pattern.compile( "^(\\d{6})\\b" ), new AccountRegularExpressionTransformer() );

        runTest( dataFile, rer );
     }

    private void runTest( String file, RegularExpressionReplacer rer ) {
       InputStream is = null;

        try {
            File f = new File( file );
            assertTrue( f.exists() && f.canRead() );
            is = new FileInputStream( f );

            is = new RegularExpressionInputStream( is, rer );

            byte [] buffer = new byte[ 2048 ];
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            int totalNumRead = 0;
            int numRead = 0;
            while( ( numRead = is.read( buffer ) )  >= 0 ) {
                totalNumRead += numRead;
                if( numRead > 0 ) {
                    baos.write( buffer, 0, numRead );
                }

                Thread.yield();
            }

            System.out.println("Results:\n" + new String( baos.toByteArray() ) );

        } catch( IOException ioe ) {
            ioe.printStackTrace();
        } finally {
            try {
                is.close();
            } catch( IOException ioe ) {
                System.out.println("Error closing data file: " + dataFile );
            }
        }

    }
}
