/***
 * Excerpted from "No Fluff Just Stuff 2006 Anthology"
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/nfjs06 for more book information.
***/
package net.bosatsu.util.regex.io;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * The RegularExpressionReplacer class represents a mapping between patterns and the objects that may
 * potentially replace text which matches each pattern.
 *
 * @author bsletten
 * @author dsletten
 *
 * @see net.bosatsu.util.regex.io.Transformer
 * @see net.bosatsu.util.regex.io.RegularExpressionInputStream
 */

public class RegularExpressionReplacer {
    private Map <Pattern, Transformer> patternMap = new HashMap <Pattern, Transformer>();

    public RegularExpressionReplacer() {
    }

    public void addPatternMapping( Pattern p, Transformer t ) {
        // If order matters, this should be stored in a FIFO linear fashion
        patternMap.put( p, t );
    }

    /*
     * Given an input string, attempt to find one or more matches
     * for the patterns associated with this RegularExpressionReplacer.
     * As matches are made, this method passes each into the transform method
     * of the corresponding Transformer instance and accumulates the results.
     *
     *  @return String representing either the original String instance if there were no matches
     *  or a transformed String if there were matches.
     *
     */

    public String performTransformations( String s ) {
        String retValue = s;
        Iterator <Pattern> itor = patternMap.keySet().iterator();

        while( itor.hasNext() ) {
            StringBuffer sb = null;
            Pattern p = itor.next();
            Matcher m = p.matcher( retValue );
            Transformer t = patternMap.get( p );

            while( m.find() ) {
                if( sb == null ) {
                    sb = new StringBuffer();
                }

                m.appendReplacement( sb, t.transform( m ) );
            }

            if( sb != null ) {
                m.appendTail( sb );
                retValue = sb.toString();
            }
        }

        return retValue;
    }
}
