/***
 * Excerpted from "No Fluff Just Stuff 2006 Anthology"
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/nfjs06 for more book information.
***/
package net.bosatsu.util.report;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.SequenceInputStream;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

/**
 * Modularize wrapping a particular InputStream with XML elements by
 * defining three separate InputStreams and serving them up to the 
 * parent SequenceInputStream.
 * 
 * Demonstrate one potential custom Decorator pattern implementation.
 * @author bsletten
 */

public class ReportInputStream extends SequenceInputStream
{
    /*
     * Constructor that takes some arbitrary String source identifier
     * and the underlying InputStream to wrap.
     */
    public ReportInputStream( String source, InputStream is ) {
        super( new StreamEnumerator( source, is ) );
    }

    /**
     * The inner class simply maintains the list of InputStreams for the
     * parent SequenceInputStream to iterate over.
     * 
     * @author bsletten
     */
    public static class StreamEnumerator implements Enumeration <InputStream> {
        private List <InputStream> streamList = new ArrayList <InputStream>();
        
        public StreamEnumerator( String source, InputStream is ) {
            // We want to wrap the underlying input stream with XML
            // elements satisfying some schema perhaps for submitting
            // to a RESTful web service. A different set of values
            // could wrap the InputStream with a SOAP request. In
            // addition to decorating existing data sources into these
            // request forms transparently, by relying on the stream
            // interfaces, we can avoid having to read the entire data
            // source into memory all at once.
            
            streamList.add( new ByteArrayInputStream( 
    		new String( "<REPORTSUMMARY><STORE>" + source + 
		"</STORE><REPORT><![CDATA[\n" ).getBytes() ) );
            streamList.add( is );
            streamList.add( new ByteArrayInputStream( 
    		new String( "\n]]></REPORT></REPORTSUMMARY>" ).getBytes() ) );
        }
        
        public boolean hasMoreElements() {
            return streamList.size() > 0;
        }

        public InputStream nextElement() {
            return streamList.remove( 0 );
        }
    }
}
