/***
 * Excerpted from "No Fluff Just Stuff 2006 Anthology"
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/nfjs06 for more book information.
***/
package net.bosatsu.util.report;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.regex.Pattern;

import junit.framework.TestCase;
import net.bosatsu.util.regex.account.AccountRegularExpressionTransformer;
import net.bosatsu.util.regex.cc.CCMaskRegularExpressionTransformer;
import net.bosatsu.util.regex.io.RegularExpressionInputStream;
import net.bosatsu.util.regex.io.RegularExpressionReplacer;

/**
 * Unit test of ReportInputStream class.
 * @author bsletten
 *
 */

public class ReportInputStream_TEST extends TestCase {
    private String dataFile;

    public void setUp() {
        dataFile = System.getProperty("user.dir")
            + File.separator + "data" + File.separator + "test.dat";
    }

    public void testBasic() {
      InputStream is = null;

        try {
            File f = new File( dataFile );
            assertTrue( f.exists() && f.canRead() );
            is = new FileInputStream( f );
            is = new ReportInputStream( "1678", is );

            runTest( is );
        } catch( IOException ioe ) {

        }
    }

    public void testCompound() {
        InputStream is = null;

          try {
              File f = new File( dataFile );
              assertTrue( f.exists() && f.canRead() );
              is = new FileInputStream( f );

              RegularExpressionReplacer rer = new RegularExpressionReplacer();
              rer.addPatternMapping( Pattern.compile( "\\b(\\d{16})\\b" ), new CCMaskRegularExpressionTransformer() );
              rer.addPatternMapping( Pattern.compile( "^(\\d{6})\\b" ), new AccountRegularExpressionTransformer() );
              is = new RegularExpressionInputStream( is, rer );
              is = new ReportInputStream( "1678", is );

              runTest( is );
          } catch( IOException ioe ) {

          }
      }

    public void runTest( InputStream is ) {
        try {
            byte [] buffer = new byte[ 2048 ];
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            int numRead = 0;

            while( ( numRead = is.read( buffer ) )  >= 0 ) {
                if( numRead > 0 ) {
                    baos.write( buffer, 0, numRead );
                }

                Thread.yield();
            }

            System.out.println("Results:\n" + new String( baos.toByteArray() ) );
        } catch( IOException ioe ) {

        }
    }
}
