/***
 * Excerpted from "No Fluff Just Stuff 2006 Anthology"
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/nfjs06 for more book information.
***/
package com.nealford.conf.domainlangs.lop;

import java.io.*;
import java.util.List;

/**
 * @author nford
 *         Date: Jun 30, 2005
 *         Time: 3:30:33 PM
 *         <cite>Incidentally, created by IntelliJ IDEA.</cite>
 *
 * Designed to handle configuration files that look like this:
 *
 * #123456789012345678901234567890123456789012345678901234567890
 * SVCLFOWLER         10101MS0120050313.........................
 * SVCLHOHPE          10201DX0320050315........................
 * SVCLTWO           x10301MRP220050329..............................
 * USGE10301TWO          x50214..7050329...............................
 */
/*START:ProcessRequest*/
public class ProcessRequest {

    public ProcessRequest() {
        createInputFile();
        Reader reader = new Reader();
        configure(reader);
        outputResults(reader.process(getInput()));
    }

    private void outputResults(List result) {
        for (Object o : result)
            System.out.println(o);
    }

    private BufferedReader getInput() {
        BufferedReader buf = null;
        try {
            buf = new BufferedReader(new FileReader("input.txt"));
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        return buf;
    }

    private void configure(Reader target) { 
        target.addStrategy(configureServiceCall());
        target.addStrategy(configueUsage());
    }

    /*START:ProcessRequest.configuration*/
    private ReaderStrategy configureServiceCall() { 
        ReaderStrategy result =
              new ReaderStrategyImpl("SVCL", ServiceCall.class);
        result.addFieldExtractor(4, 18, "customerName");
        result.addFieldExtractor(19, 23, "customerId");
        result.addFieldExtractor(24, 27, "callTypeCode");
        result.addFieldExtractor(28, 35, "dateOfCallString");
        return result;
    }

    private ReaderStrategy configueUsage() { 
        ReaderStrategy result =
              new ReaderStrategyImpl("USGE", Usage.class);
        result.addFieldExtractor(4, 8, "customerId");
        result.addFieldExtractor(9, 22, "customerName");
        result.addFieldExtractor(23, 30, "cycle");
        result.addFieldExtractor(31, 36, "readDate");
        return result;
    }
    /*END:ProcessRequest.configuration*/

    public static void main(String[] args) {
        new ProcessRequest();
    }
}
/*END:ProcessRequest*/
