/***
 * Excerpted from "No Fluff Just Stuff 2006 Anthology"
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/nfjs06 for more book information.
***/
package com.nealford.conf.domainlangs.lop;

import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import java.io.BufferedReader;
import java.io.IOException;

/**
 * @author nford
 *         Date: Jun 30, 2005
 *         Time: 2:36:21 PM
 *         <cite>Incidentally, created by IntelliJ IDEA.</cite>
 */
/*START:Reader*/
public class Reader {
    private Map<String, ReaderStrategy> _strategies =
            new HashMap<String, ReaderStrategy>();

    public List<String> process(BufferedReader input) {
        List<String> result = new ArrayList<String>();
        String line;
        try {
            while ((line = input.readLine()) != null)
                processLine(line, result);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return result;
    }

    private void processLine(String line, List result) {
        if (isBlank(line)) return;
        if (isComment(line)) return;
        String typeCode = getTypeCode(line);
        ReaderStrategy strategy = _strategies.get(typeCode);
        if (null == strategy)
            throw new RuntimeException("Unable to find suitable strategy");
        result.add(strategy.process(line));
    }

    private static boolean isComment(String line) {
        return line.charAt(0) == '#';
    }

    private static boolean isBlank(String line) {
        return line.equals("");
    }

    private String getTypeCode(String line) {
        return line.substring(0, 4);
    }

    public void addStrategy(ReaderStrategy strategy) {
        _strategies.put(strategy.getCode(), strategy);
    }

}
/*END:Reader*/
