/***
 * Excerpted from "No Fluff Just Stuff 2006 Anthology"
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/nfjs06 for more book information.
***/
package com.nealford.conf.domainlangs.lop;

import java.util.List;
import java.util.ArrayList;
import java.lang.reflect.Field;

/**
 * @author nford
 *         Date: Jun 30, 2005
 *         Time: 2:44:31 PM
 *         <cite>Incidentally, created by IntelliJ IDEA.</cite>
 */
/*START:ReaderStrategyImpl*/
public class ReaderStrategyImpl implements ReaderStrategy {
    private String _code;
    private Class _target;

    final class FieldExtractor { // <callout id="co.FieldExtractor" />
        private int _begin, _end;
        private String _targetPropertyName;

        public FieldExtractor(int begin, int end, String target) {
            _begin = begin;
            _end = end;
            _targetPropertyName = target;
        }

        public void extractField(String line, Object targetObject) { // <callout id="co.extractField" />
            String value = line.substring(_begin, _end + 1);
            try {
                setValue(targetObject, value);
            } catch (Exception oops) {
                throw new RuntimeException(oops);
            }
        }

        private void setValue(Object targetObject, String value) // <callout id="co.setValue" />
	    throws Exception {
            Field field = targetObject.getClass().getField(_targetPropertyName);
            field.set(targetObject, value);
        }
    }

    private List<ReaderStrategyImpl.FieldExtractor> extractors =
            new ArrayList<ReaderStrategyImpl.FieldExtractor>(); // <callout id="co.listOfExtractors" />

    public ReaderStrategyImpl(String code, Class target) {
        _code = code;
        _target = target;
    }

    public String getCode() {
        return _code;
    }

    public void addFieldExtractor(int begin, int end, String target) {
        if (! targetPropertyNames().contains(target))
            throw new NoFieldInTargetException(target, _target.getName());
        extractors.add(new ReaderStrategyImpl.FieldExtractor(begin, end, target));
    }

    private List<String> targetPropertyNames() {
        List<String> result = new ArrayList<String>();
        Field fields[] = _target.getFields();
        for (Field f : fields)
            result.add(f.getName());
        return result;
    }

    public Object process(String line) {
        Object result = null;
        try {
            result = _target.newInstance();
        } catch (Exception oops) {
            oops.printStackTrace();
        }
        for (FieldExtractor ex : extractors) // <callout id="co.loopOverExtractors" />
            ex.extractField(line, result);
        return result;
    }
}
/*END:ReaderStrategyImpl*/
